/*
	eeprom.c --- Serial EEPROM read/write function
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>

#include "user_define.h"

/*
 * write
 */
int eeprom_wr(unsigned int adr, unsigned char *dataptr, unsigned int bytes)
{
	unsigned char buff[EEPROM_MAX_BYTE + 2];
	int p;
	int ret;

	buff[0] = (unsigned char)(adr >> 8);	  // upper byte
	buff[1] = (unsigned char)(adr & 0x00ff);   // lower byte
	for (p=0; p < bytes; p++){
		buff[p + 2] = *dataptr;
		dataptr++;
	}
	ret = i2c_send(I2C_ADR_EEPROM, buff, bytes + 2);
	return ret;
}
/*
 * read
 */
int eeprom_rd(unsigned int adr, unsigned char *dataptr, unsigned int bytes)
{
	unsigned char buff[2];
	int ret;

	buff[0] = (unsigned char)(adr >> 8);	  // upper byte
	buff[1] = (unsigned char)(adr & 0x00ff);   // lower byte

	ret = i2c_send(I2C_ADR_EEPROM, buff, 2);	// send address
	if (ret == 2){	// OK
		ret = i2c_recv(I2C_ADR_EEPROM, dataptr, bytes);
	}
	return ret;
}

// eeprom read/write test
// cmdline format:
//    >$ rAAAA  -> eeprom read
//    >$ wAAAA DD DD DD ... -> eeprom write
void eeprom_test1(unsigned char *param)
{
	unsigned int top_adr;
	unsigned char buff[18];
	char str[8];
	unsigned char *cp;
	int j,k, ret;

	if (param[0] == 'w'){
		cp = ++param;
		top_adr = hex2val(&cp);	// address in hex
		k = 0;
		while ((*cp != NULL) && (k < 0x10)){ // get each data in list
			buff[k] = hex2val(&cp);
			k++;
		}
		ret = eeprom_wr(top_adr, buff, k);
		uart_sendstr("eeprom write:\r\n");
		hexstr(top_adr, str);
		uart_sendstr("address = 0x");
		uart_sendstr(str);
		uart_sendstr("\r\n");
		for (j=0; j<k; j++){
				hexstr(buff[j], str);
				uart_sendstr(" ");
				uart_sendstr(&str[2]);
		}
		uart_sendstr("\r\n");
		uart_sendstr("return of eeprom_wr() = 0x");
		hexstr(ret, str);		uart_sendstr(str);
		uart_sendstr("\r\n");
	}
	if (param[0] == 'r'){
		cp = ++param;
		top_adr = hex2val(&cp);	// address in hex
		ret = eeprom_rd(top_adr, buff, 16);
		uart_sendstr("eeprom read:\r\n");
		hexstr(top_adr, str);
		uart_sendstr("address = 0x");
		uart_sendstr(str);
		uart_sendstr("\r\n");
		for (j=0; j<16; j++){
				hexstr(buff[j], str);
				uart_sendstr(" ");
				uart_sendstr(&str[2]);
		}
		uart_sendstr("\r\n");
		uart_sendstr("return of eeprom_rd() = 0x");
		hexstr(ret, str);		uart_sendstr(str);
		uart_sendstr("\r\n");
	}

}
