/*
	temp_sensor.c --- Temperature sensor (STTS751-0WB) function
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <math.h>

#include "user_define.h"



/*
 * initialize tempe sensor
 */
int init_temp_sensor()
{
	unsigned char buff[8];
	int ret;

	buff[0] = 0x03;		// config reg. addr
	buff[1] = 0x0c;		// bit[4:3]=11  12bit, unit: 0.0625 degree
	ret = i2c_send(I2C_ADR_TEMPSNS, buff, 2);
	if (ret != 2) return ret;

	buff[0] = 0x04;		// conversion rate reg. addr
	buff[1] = 0x04;		// bit[3:0]=4   1/sec. conversion
	ret = i2c_send(I2C_ADR_TEMPSNS, buff, 2);
	if (ret != 2) return ret;

	return ret;
}
/*
 * get temperature
 *   return: raw 2byte data from sensor register addr. 0x00 and 0x02.
 */
unsigned int get_temperature()
{
	unsigned char buff[8];
	int ret, result, error, errlog[4];

	error = 0;
	//errlog[0] = 0;	errlog[1] = 0;	errlog[2] = 0;	errlog[3] = 0;
	buff[0] = 0x00;		// reg. addr
	ret = i2c_send(I2C_ADR_TEMPSNS, buff, 1);
	if (ret != 1) { error |= 0x01;	errlog[0] = ret; }
	ret = i2c_recv(I2C_ADR_TEMPSNS, buff, 1);
	if (ret != 1) { error |= 0x02;	errlog[1] = ret; }
	result = (unsigned int)(buff[0] << 8);

	buff[0] = 0x02;		// reg. addr
	ret = i2c_send(I2C_ADR_TEMPSNS, buff, 1);
	if (ret != 1) { error |= 0x04;	errlog[2] = ret; }
	ret = i2c_recv(I2C_ADR_TEMPSNS, buff, 1);
	if (ret != 1) { error |= 0x08;	errlog[3] = ret; }
	result += buff[0];

	if (error) return error;
	else return result;
}

/*
 * convert raw data to [-]XX.XXXX degree string
 */
void raw2degree(unsigned int rawdata, char *str)
{
	int int_part, frac_part, frac10k;
	int sign,p;
	int u;

	p = 0;
	sign = 0;
	int_part = (rawdata >> 8);
	frac_part = (rawdata & 0x00ff) >> 4;

	if (int_part & 0x80){
		str[p++] = '-';
		int_part = (~int_part + 1) & 0xFF;	// 2's complement, 8bit only
		frac_part = ~frac_part + 1;	// 2's complement
		sign = 1;
	}
	str[p++] = '0' + (int_part / 100);	int_part = int_part % 100;
	str[p++] = '0' + (int_part / 10);	int_part = int_part % 10;
	str[p++] = '0' + int_part;

	u = 625;	// 0.0625 x 10000
	frac10k = 0;
	if (frac_part & 0x0001){	frac10k += u;	}
	u *= 2;
	if (frac_part & 0x0002){	frac10k += u;	}
	u *= 2;
	if (frac_part & 0x0004){	frac10k += u;	}
	u *= 2;
	if (frac_part & 0x0008){	frac10k += u;	}

	str[p++] = '.';
	str[p++] = '0' + (frac10k / 1000);	frac10k = frac10k % 1000;
	str[p++] = '0' + (frac10k / 100);	frac10k = frac10k % 100;
	str[p++] = '0' + (frac10k / 10);	frac10k = frac10k % 10;
	str[p++] = '0' + frac10k;
	str[p] = 0;
}

/*
 * DEBUG
 */
void temp_test1()
{
	unsigned int temp;
	char str[10];

	temp = get_temperature();
	hexstr(temp, str);
	uart_sendstr("temp. = 0x");
	uart_sendstr(str);
	uart_sendstr(", ");
	raw2degree(temp, str);
	uart_sendstr(str);
	uart_sendstr(" degree\r\n");
}
