/*
	uart.c --- UART (Serial) interface functions
*/

#include <msp430.h>
#include <stdio.h>

#include "user_define.h"

/* -------- global var. for serial port -------- */
unsigned char UART_RXDBUF[UART_RXBUFMAX];
int UART_RCV_PTR, UART_READ_PTR, UART_RCV_LEN;


/*
 * init UART port
 */
void init_uart()
{
	// switch P1.1, P1.2 as UART RXD,TXD pin
	P1REN &= ~(BIT1 | BIT2);	// pull-u/d R disable
	P1SEL  |= (BIT1 | BIT2);	// to UART
	P1SEL2 |= (BIT1 | BIT2);	// to UART
	USCI_A0_graceInit();
	IE2 |= UCA0RXIE;		// uart receive interrupt enable
}

/*
 * close UART port
 */
void close_uart()
{
	UCA0CTL1 |= UCSWRST;	// disable USCI
	// switch P1.1, P1.2 as GPIO input pin
	P1SEL &= ~(BIT1 | BIT2);	// to GPIO
	P1SEL2 &= ~(BIT1 | BIT2);	// to GPIO
	P1DIR &= ~(BIT1 | BIT2);	// input
	P1REN |= (BIT1 | BIT2);		// pull-u/d R enable
	P1OUT &= ~(BIT1 | BIT2);	// pull-down
}

// Send a char on TxD
int uart_sendchar(unsigned char data)
{
  unsigned int tmout;
  tmout=5000;
  while (!(IFG2 & UCA0TXIFG) && (tmout--))	// Poll TXIFG to until set
	  ;
  if (tmout == 0){
    return 1;
  } else {
    UCA0TXBUF = data;                    // TX -> RXed character
  }
  return 0;
}

// Send string to serial console
void uart_sendstr(char *s)
{
  while (*s != NULL){
    uart_sendchar((unsigned char)(*s++));
  }
}


// get a rxd byte from buffer
unsigned char uart_getchar(){
  unsigned char rxdata;
  if (UART_RCV_LEN > 0){
    UART_RCV_LEN--;
    rxdata = UART_RXDBUF[UART_READ_PTR];
    if (UART_READ_PTR < (UART_RXBUFMAX-1)){
      UART_READ_PTR++;
    } else {
      UART_READ_PTR = 0;    // ptr increment s cyclic
    }
  } else {  // no data
  }
  return rxdata;
}
// check received bytes
unsigned int recv_len(){
  return UART_RCV_LEN;
}


