/*
	uart_session.c --- user session via UART (Serial) interface functions
*/

#include <msp430.h>
#include <stdio.h>
#include <ctype.h>

#include "user_define.h"

/* -------- global var. -------- */
unsigned char CMDLINE[CMDLINE_BUFSIZE];

/* check a char within '0'-'9' */
BOOL isNumber(unsigned char c){
	BOOL ret;
	ret = FALSE;
	if ((c>='0') && (c<='9'))  ret = TRUE;
	return ret;
}

/* check a char within '0'-'9' 'a'-'f' 'A'-'F' */
BOOL isHexChar(unsigned char c){
	BOOL ret;
	ret = FALSE;
	if ((c>='0') && (c<='9'))  ret = TRUE;
	c = toupper(c);
	if ((c>='A') && (c<='F'))  ret = TRUE;
	return ret;
}

/* number string to int (up to 5digit)*/
/* find number char, get value.       */
/* (value should end Space or null(0)) */
unsigned int num2val(unsigned char **ptr)
{
	unsigned int m,c, ret;
	unsigned char *ptr_next;
	c=0;
	ret=0;

	while (!isNumber(**ptr)){ // forward to hex char.
		if (**ptr == NULL) return 0;
		++*ptr;
	}
	while ((**ptr!=NULL)&&(**ptr!=SPC)&&(c<5)){ 	// forward to delimiter
		++*ptr;
		c++;
	}
	if (c>=6){ return 0; }
	m = 1;
	ptr_next = *ptr;
	while (c>0){
		--*ptr;
		ret += m * (**ptr -'0');
		m *= 10;
		c--;
	}
	*ptr = ptr_next;	// the pointer forward to the next delimiter
	return ret;
}


/* hex string to int (up to 4digit)*/
/* find hex char, get value.       */
/* (value should end Space or null(0)) */
unsigned int hex2val(unsigned char **ptr)
{
	unsigned int m,c, ret;
	unsigned char *ptr_next;
	c=0;
	ret=0;

	while (!isHexChar(**ptr)){ // forward to hex char.
		if (**ptr == NULL) return 0;
		++*ptr;
	}
	while ((**ptr!=NULL)&&(**ptr!=SPC)&&(c<5)){ 	// forward to delimiter
		++*ptr;
		c++;
	}
	if (c>=5){ return 0; }
	m = 1;
	ptr_next = *ptr;
	while (c>0){
		--*ptr;
		if (toupper(**ptr) >= 'A'){
			ret += m * ((toupper(**ptr)-'A') + 10);
		} else {
			ret += m * (**ptr -'0');
		}
		m = m << 4;
		c--;
	}
	*ptr = ptr_next;	// the pointer forward to the next delimiter
	return ret;
}

/* int to number string (up to 5digit)*/
void numstr(unsigned int val, char *str)
{
	str[5] = 0;
	str[4] = '0' + (val % 10);	val /= 10;
	str[3] = '0' + (val % 10);	val /= 10;
	str[2] = '0' + (val % 10);	val /= 10;
	str[1] = '0' + (val % 10);	val /= 10;
	str[0] = '0' + (val % 10);	val /= 10;
}

/* int to hex string (up to 4digit)*/
void hexstr(unsigned int val, char *str)
{
	char *hextbl="0123456789ABCDEF?";
	str[0] = hextbl[val >> 12];
	str[1] = hextbl[(val >> 8) & 0x000f];
	str[2] = hextbl[(val >> 4) & 0x000f];
	str[3] = hextbl[val & 0x000f];
	str[4] = 0;
}
/* int to hex string (up to 8digit)*/
void hexstr32(unsigned long int val, char *str)
{
	char *hextbl="0123456789ABCDEF?";
	str[0] = hextbl[val >> 28];
	str[1] = hextbl[(val >> 24) & 0x0000000f];
	str[2] = hextbl[(val >> 20) & 0x0000000f];
	str[3] = hextbl[(val >> 16) & 0x0000000f];
	str[4] = hextbl[(val >> 12) & 0x0000000f];
	str[5] = hextbl[(val >> 8) & 0x0000000f];
	str[6] = hextbl[(val >> 4) & 0x0000000f];
	str[7] = hextbl[val & 0x0000000f];
	str[8] = 0;
}

/* ---------- Terminal session functions ---------- */
void show_help()
{
	char *cp =
		"----(Logging/Record command)--------\r\n"
		" log (1|0)      : Logging 1:start, 0:stop\r\n"
		" ld             : dump recorded data\r\n"
		" ls             : show logging status\r\n"
		" lc (0|1|2|3|?) : change recording config. or ?:help\r\n"
		" ll             : list records\r\n"
		" lp <adr(hex)>  : set address pointer\r\n"
		" li <count>     : set item count\r\n"
		" lem            : erase all memory\r\n"
		"----(RTC command)--------\r\n"
		" R              : show RTC master counter and XCAP\r\n"
		" t              : show RTC in y/m/d h:m:s\r\n"
		" % <year> <month> <day> <hour> <minutes> <sec>: set RTC master counter\r\n"
		" XC (0|1|2|3)   : change XCAP setting\r\n"
		"----(I2C command)--------\r\n"
		" a <adr(hex)>   : set slave address\r\n"
		" s xx [xx] ..   : set byte(s) data to write\r\n"
		" c <bytes(hex)> : set count bytes to read\r\n"
		" r              : read access\r\n"
		" w              : write access\r\n"
		" p              : show current parameters\r\n"
		" 0              : reset i2c I/F\r\n"
		" P              : output stop condition\r\n"
		" -              : check SCL,SDA level\r\n"
		"read/write bytes count is 64 bytes max.\r\n"
		"----(other command)--------\r\n"
		" ?              : show help\r\n"
		" @       : (unit test for debug)\r\n"
		" ~       : read temperature (unit test for debug)\r\n"
		"";
	uart_sendstr(cp);
}


/* --- get one command line --- */
/*   return  >=0 : length (enter key pressed)
             <0  : no any key hit
     always through and return
*/
int cmd_line(unsigned char *cmdlineptr)
{
	unsigned char cin;
	int len;
	BOOL enterkey;
	static int cptr = 0;

	enterkey = FALSE;
	while(recv_len() > 0){
		if (cin = uart_getchar()){
			if (cin == BS){
				if (cptr >0){
					uart_sendchar(BS);
					uart_sendchar(' ');
					uart_sendchar(BS);
					cptr--;
				} else {
					uart_sendchar(BEL);
				}
			}
			else if (cin == CR){
				uart_sendchar(CR);
				uart_sendchar(LF);
				cmdlineptr[cptr] = NULL;	// ends by NULL
				len = cptr;
				cptr = 0;
				enterkey = TRUE;
			}
			else {
				if (cptr < (CMDLINE_BUFSIZE-1)){
					uart_sendchar(cin);
					cmdlineptr[cptr++] = cin;
				} else {
					uart_sendchar(BEL);
				}
			}
		}
	} /* while(.. */
	if (enterkey){
		return len;
	} else {
		return -1;
	}
}


/* decode and process command ----- i2c */
void process_command_i2c(unsigned char *cmdline)
{
	int j,cnt, ret;
	static int txlen=1, rxlen=1;
	unsigned char *cp;
	char str[10];
	unsigned int adr,data;
	BOOL b_scl,b_sda;

	switch (cmdline[0]){
		case 'a':			// set slave address
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				adr = hex2val(&cp);
				I2C_SLAVE_ADR = adr & 0x00ff;
				hexstr(I2C_SLAVE_ADR, str);
				uart_sendstr("Set slave address = 0x");
				uart_sendstr(&str[2]);
				uart_sendstr("\r\n");
			}
			break;
		case 'c':			// set read bytes
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				j = hex2val(&cp);
				if (j > I2C_BUFSIZE){
					uart_sendstr("over range\r\n");
				} else {
					rxlen = j;
					hexstr(rxlen, str);
					uart_sendstr("read counter = 0x");
					uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
				}
			}
			break;
		case 's':			// set write data array
			cp = ++cmdline;
			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;
			if (*cp == NULL){
				uart_sendstr("?\r\n");
			} else {
				cnt = 0;
				while (cnt < I2C_BUFSIZE){
					data = hex2val(&cp);
					I2C_BUF_W[cnt++] = (unsigned char)(data & 0x00ff);
//			while ((*cp!=NULL) && isHexChar(*cp)) cp++;	// skip current data str
//			while ((*cp!=NULL) && !isHexChar(*cp)) cp++;	// skip space
					if (*cp == NULL) break;
				}
				txlen = cnt;
				uart_sendstr("data for write (hex):");
				for (j=0; j<txlen; j++){
					hexstr(I2C_BUF_W[j], str);
					uart_sendstr(&str[2]);
					uart_sendstr(" ");
				}
				uart_sendstr(",  0x");
				hexstr(txlen, str);
				uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			}
			break;
		case 'p':			// show current parameters
			hexstr(I2C_SLAVE_ADR, str);
			uart_sendstr("slave address = 0x");
			uart_sendstr(&str[2]);
			uart_sendstr("\r\n");
			hexstr(rxlen, str);
			uart_sendstr("read counter = 0x");		uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			uart_sendstr("write data: ");
			for (j=0; j<txlen; j++){
				hexstr(I2C_BUF_W[j], str);
				uart_sendstr(&str[2]);
				uart_sendstr(" ");
			}
			uart_sendstr(",  0x");
			hexstr(txlen, str);			uart_sendstr(&str[2]);		uart_sendstr(" bytes\r\n");
			break;
		case 'w':			// execute i2c write
			I2C_TxByteCnt = txlen;
			ret = i2c_send(I2C_SLAVE_ADR, I2C_BUF_W, I2C_TxByteCnt);
			if (ret == E_BUSY)		uart_sendstr("BUSY\r\n");
			if (ret == E_NACK)		uart_sendstr("NACK\r\n");
			if (ret == E_TMOUT)		uart_sendstr("Time out\r\n");
			break;
		case 'r':			// execute i2c read
			I2C_RxByteCnt = rxlen;
			ret = i2c_recv(I2C_SLAVE_ADR, I2C_BUF_R, I2C_RxByteCnt);
			if (ret == E_BUSY)		uart_sendstr("BUSY\r\n");
			if (ret == E_NACK)		uart_sendstr("NACK\r\n");
			if (ret == E_TMOUT)		uart_sendstr("Time out\r\n");
			if (ret > 0){
				for (j=0; j<ret; j++){
					hexstr(I2C_BUF_R[j], str);
					uart_sendstr(&str[2]);
					uart_sendstr(" ");
					if (((j+1) & 0x000f) == 0)	uart_sendstr("\r\n");
				}
				uart_sendstr("\r\n");

			}
			break;
		case '0':			// i2c reset
			i2c_reset();
			uart_sendstr("I2C reset\r\n");
			break;
		case 'P':			// i2c stop condition
			i2c_stop();
			uart_sendstr("stop condition\r\n");
			break;
		case '-':			// check i2c SCL,SDA signal
			i2c_siglevel(&b_scl, &b_sda);
			uart_sendstr("SCL=");
			if (b_scl)	uart_sendstr("HIGH");	else	uart_sendstr("LOW");
			uart_sendstr(" SDA=");
			if (b_sda)	uart_sendstr("HIGH");	else	uart_sendstr("LOW");
			uart_sendstr("\r\n");
			break;

		default:
			break;
	}

}

/* decode and process command ----- RTC */
void process_command_rtc(unsigned char *cmdline)
{
	unsigned char *cp;
	char str[10];
	int year;
	char mon,day,hour,min,sec;
	DATETYPE date;
	int ret;

	switch (cmdline[0]){
		case 'R':			// show RTC counter value
			hexstr32(RTC_TIME_COUNTER, str);
			uart_sendstr("RTC_TIME_COUNTER = 0x");
			uart_sendstr(str);
			uart_sendstr("  (XCAP=");
			uart_sendchar('0' + get_xcap_setting());
			uart_sendstr(")\r\n");
			break;
		case 't':			// show RTC in y/m/d h:m:s
			date = rtc_count2ymd(RTC_TIME_COUNTER);
			show_date_time(date);
			uart_sendstr("\r\n");
			led_2(FALSE);
			break;
		case '%':			// set RTC counter value
			cp = cmdline;
			cp++;
			year = num2val(&cp);
			mon = num2val(&cp);
			day = num2val(&cp);
			hour = num2val(&cp);
			min = num2val(&cp);
			sec = num2val(&cp);
			ret = set_rtc_counter(year, mon, day, hour, min, sec);
			uart_sendstr((char *)&cmdline[2]);
			uart_sendstr(" offset ");
			if (ret < 0){	ret = -ret;	uart_sendchar('-');	}
			numstr(ret, str);
			uart_sendstr(str);	uart_sendstr(" sec\r\n");
			led_2(FALSE);
			break;
		case 'X':			// change XCAP setting
			if (cmdline[1] == 'C'){	// 'X' and 'C'
				set_xcap_setting(cmdline[3] - '0');
				uart_sendstr("new ");
			}
			uart_sendstr("XCAP=");
			uart_sendchar('0' + get_xcap_setting());
			uart_sendstr("\r\n");
			break;
	}
}

/* decode and process command ----- recording (logging) */
void process_command_record(unsigned char *cmdline)
{
	unsigned char *cp;
	char str[10];
	unsigned int adr, param1;

	if (cmdline[0] != 'l') return;

	switch (cmdline[1]){
		case 'o' :			// logging start/stop
			cp = &cmdline[2];
			param1 = num2val(&cp);
			if (param1 == 1)
				set_rec_mode(REC_MODE_RUN);
			else
				set_rec_mode(REC_MODE_STOP);
			break;
		case 'd' :			// dump log
			if (REC_MODE == REC_MODE_RUN)
				dump_record(REC_START_ADR_PTR, REC_ITEMS);
			else
				dump_record(REC_START_ADR_PTR, 0);
			break;
		case 's' :			// show logging status
			show_current_rec_status();
			break;
		case 'c' :			// config setting or show help
			cp = &cmdline[2];
			param1 = num2val(&cp);
			if (cmdline[3] == '?'){
				uart_sendstr(" Config option No. settings:\r\n");
				show_config_options();
			}
			else if ((param1 >= 0) && (param1 <= 3)){	//valid parameter
				if (REC_MODE == REC_MODE_RUN){
					uart_sendstr(" Cannot be changed the config in RUN mode.\r\n");
				}
				else {
					set_rec_config(param1);
					uart_sendstr(" set record config: ");
					uart_sendchar('0' + REC_CONFIG_MODE);
					uart_sendstr("\r\n");
				}
			}
			break;
		case 'l' :		// list records
			record_list_select();
			break;
		case 'p' :		// set address pointer
			if ((cmdline[2] == ' ') && (isHexChar(cmdline[3]))){
				cp = &cmdline[2];
				adr = hex2val(&cp);
				if (set_adr_ptr(adr)){
					uart_sendstr("Ok, new address pointer set.\r\n");
					REC_ITEMS = 0;
				}
				else
					uart_sendstr("illegal value\r\n");
			}
			else	show_current_rec_status();
			break;
		case 'i' :		// set item counter
			cp = &cmdline[2];
			REC_ITEMS = num2val(&cp);
			break;
		case 'e' :		// erase all memory
			if (cmdline[2] != 'm') break;		// check 'lem'
			erase_all_memory();
			break;
		default:
			break;
	}
}

/*
 *  decode and process command ----- main
 */
void process_command(unsigned char *cmdline)
{
	int j;
	//char str[10];


	switch (cmdline[0]){
		case '?':
			show_help();
			break;

		case 'a':		// set slave address
		case 'c':		// set read bytes
		case 's':		// set write data array
		case 'p':		// show current parameters
		case 'w':		// execute i2c write
		case 'r':		// execute i2c read
		case '0':		// i2c reset
		case 'P':		// i2c stop condition
		case '-':		// check i2c SCL,SDA signal
			process_command_i2c(cmdline);
			break;

		case 'R':		// show RTC counter value
		case 't':		// show RTC in y/m/d h:m:s
		case '%':		// set RTC counter value
		case 'X':		// change XCAP setting
			process_command_rtc(cmdline);
			break;

		case 'l':		// logging related command; lo,ld,ls,lc,ll,lp,li,le
			if ((cmdline[1] != 'o') &&
				(cmdline[1] != 'd') &&
				(cmdline[1] != 's') &&
				(cmdline[1] != 'c') &&
				(cmdline[1] != 'l') &&
				(cmdline[1] != 'p') &&
				(cmdline[1] != 'i') &&
				(cmdline[1] != 'e')
				)	break;
			process_command_record(cmdline);
			break;

		case '@':			// unit test //DEBUG
			uart_sendstr("enter PM_MODE_LOW \r\n");
			//close_uart();
			//P1OUT &= ~BIT0;	// LED1 Off
			set_pmmode(PM_MODE_LOW);
			j=30000;
			while (j--)
				;
			//init_uart();
			//P1OUT |= BIT0;	// LED1 On
			uart_sendstr("\r\n returned. \r\n");
			break;
		case '$':			// unit test //DEBUG
			// cmdline format:
			//    >$ rAAAA  -> eeprom read
			//    >$ wAAAA DD DD DD ... -> eeprom write
			eeprom_test1(&cmdline[2]);
			break;
		case '~':			// unit test -- read temp //DEBUG
			temp_test1();
			break;

		default:
				break;
	}
}

/* Session --- if no received data, then through and return */
void session()
{
	char str[10];//DEBUG

	if (UART_RCV_LEN < 0){		//DEBUG
		fatal_error(2, 1, "session");
	}	// DEBUG

	if (cmd_line(CMDLINE) >= 0){
		process_command(CMDLINE);

		numstr(REC_ITEMS, str);
		uart_sendstr("Rec:");
		if (REC_MODE == REC_MODE_RUN)
			uart_sendstr("RUN");
		else
			uart_sendstr("STOP");
		uart_sendstr("[");
		uart_sendstr(str);
		uart_sendstr("]>");
/*
		hexstr32(RTC_TIME_COUNTER, str);
		uart_sendstr("T=");
		uart_sendstr(str);
		uart_sendstr("h>");
*/
		any_activity();	// touch
	}
	else {
		;		// quit immediately.
	}
}
